@@ -25,15 +25,17 @@
  */
 
 #include "dssp-io.hpp"
+
+#include "cif++/category.hpp"
+#include "cif++/validate.hpp"
 #include "revision.hpp"
 
+#include <algorithm>
 #include <cif++.hpp>
 #include <cif++/dictionary_parser.hpp>
-
-#include <exception>
-#include <filesystem>
-#include <fstream>
+#include <format>
 #include <iostream>
+#include <type_traits>
 
 // --------------------------------------------------------------------
 
@@ -121,20 +123,20 @@ std::string ResidueToDSSPLine(const dssp::residue_info &info)
 		if (acceptor)
 		{
 			auto d = acceptor.nr() - info.nr();
-			NHO[i] = cif::format("{:d},{:3.1f}", d, acceptorE);
+			NHO[i] = std::format("{:d},{:3.1f}", d, acceptorE);
 		}
 
 		if (donor)
 		{
 			auto d = donor.nr() - info.nr();
-			ONH[i] = cif::format("{:d},{:3.1f}", d, donorE);
+			ONH[i] = std::format("{:d},{:3.1f}", d, donorE);
 		}
 	}
 
 	// auto ca = residue.atomByID("CA");
 	auto const &[cax, cay, caz] = residue.ca_location();
 
-	return cif::format("{:5d}{:5d}{:1.1s}{:1.1s} {:1c}  {:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:4d}{:4d}{:1c}{:4.0f} {:>11s}{:>11s}{:>11s}{:>11s}  {:6.3f}{:6.1f}{:6.1f}{:6.1f}{:6.1f} {:6.1f} {:6.1f} {:6.1f}",
+	return std::format("{:5d}{:5d}{:1.1s}{:1.1s} {:1c}  {:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:1c}{:4d}{:4d}{:1c}{:4.0f} {:>11s}{:>11s}{:>11s}{:>11s}  {:6.3f}{:6.1f}{:6.1f}{:6.1f}{:6.1f} {:6.1f} {:6.1f} {:6.1f}",
 		info.nr(), residue.pdb_seq_num(), residue.pdb_ins_code(), residue.pdb_strand_id(), code,
 		ss, helix[3], helix[0], helix[1], helix[2], bend, chirality, bridgelabel[0], bridgelabel[1],
 		bp[0], bp[1], sheet, floor(info.accessibility() + 0.5),
@@ -167,40 +169,40 @@ void writeDSSP(const dssp &dssp, std::ostream &os)
 	   << dssp.get_pdb_header_line(dssp::pdb_record_type::SOURCE) << '.' << std::endl
 	   << dssp.get_pdb_header_line(dssp::pdb_record_type::AUTHOR) << '.' << std::endl;
 
-	os << cif::format("{:5d}{:3d}{:3d}{:3d}{:3d} TOTAL NUMBER OF RESIDUES, NUMBER OF CHAINS, NUMBER OF SS-BRIDGES(TOTAL,INTRACHAIN,INTERCHAIN)                .",
+	os << std::format("{:5d}{:3d}{:3d}{:3d}{:3d} TOTAL NUMBER OF RESIDUES, NUMBER OF CHAINS, NUMBER OF SS-BRIDGES(TOTAL,INTRACHAIN,INTERCHAIN)                .",
 			  stats.count.residues, stats.count.chains, stats.count.SS_bridges, stats.count.intra_chain_SS_bridges, (stats.count.SS_bridges - stats.count.intra_chain_SS_bridges))
 	   << std::endl;
 
-	os << cif::format("{:8.1f}   ACCESSIBLE SURFACE OF PROTEIN (ANGSTROM**2)                                                                         .", stats.accessible_surface) << std::endl;
+	os << std::format("{:8.1f}   ACCESSIBLE SURFACE OF PROTEIN (ANGSTROM**2)                                                                         .", stats.accessible_surface) << std::endl;
 
 	// hydrogenbond summary
 
-	os << cif::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS OF TYPE O(I)-->H-N(J)  , SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds, (stats.count.H_bonds * 100.0 / stats.count.residues)) << std::endl;
+	os << std::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS OF TYPE O(I)-->H-N(J)  , SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds, (stats.count.H_bonds * 100.0 / stats.count.residues)) << std::endl;
 
-	os << cif::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS IN     PARALLEL BRIDGES, SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds_in_parallel_bridges, (stats.count.H_bonds_in_parallel_bridges * 100.0 / stats.count.residues)) << std::endl;
+	os << std::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS IN     PARALLEL BRIDGES, SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds_in_parallel_bridges, (stats.count.H_bonds_in_parallel_bridges * 100.0 / stats.count.residues)) << std::endl;
 
-	os << cif::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS IN ANTIPARALLEL BRIDGES, SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds_in_antiparallel_bridges, (stats.count.H_bonds_in_antiparallel_bridges * 100.0 / stats.count.residues)) << std::endl;
+	os << std::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS IN ANTIPARALLEL BRIDGES, SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_bonds_in_antiparallel_bridges, (stats.count.H_bonds_in_antiparallel_bridges * 100.0 / stats.count.residues)) << std::endl;
 
 	for (int k = 0; k < 11; ++k)
-		os << cif::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS OF TYPE O(I)-->H-N(I{:1c}{:1d}), SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_Bonds_per_distance[k], (stats.count.H_Bonds_per_distance[k] * 100.0 / stats.count.residues), (k - 5 < 0 ? '-' : '+'), abs(k - 5)) << std::endl;
+		os << std::format("{:5d}{:5.1f}   TOTAL NUMBER OF HYDROGEN BONDS OF TYPE O(I)-->H-N(I{:1c}{:1d}), SAME NUMBER PER 100 RESIDUES                              .", stats.count.H_Bonds_per_distance[k], (stats.count.H_Bonds_per_distance[k] * 100.0 / stats.count.residues), (k - 5 < 0 ? '-' : '+'), abs(k - 5)) << std::endl;
 
 	// histograms...
 	os << "  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30     *** HISTOGRAMS OF ***           ." << std::endl;
 
 	for (auto hi : stats.histogram.residues_per_alpha_helix)
-		os << cif::format("{:3d}", hi);
+		os << std::format("{:3d}", hi);
 	os << "    RESIDUES PER ALPHA HELIX         ." << std::endl;
 
 	for (auto hi : stats.histogram.parallel_bridges_per_ladder)
-		os << cif::format("{:3d}", hi);
+		os << std::format("{:3d}", hi);
 	os << "    PARALLEL BRIDGES PER LADDER      ." << std::endl;
 
 	for (auto hi : stats.histogram.antiparallel_bridges_per_ladder)
-		os << cif::format("{:3d}", hi);
+		os << std::format("{:3d}", hi);
 	os << "    ANTIPARALLEL BRIDGES PER LADDER  ." << std::endl;
 
 	for (auto hi : stats.histogram.ladders_per_sheet)
-		os << cif::format("{:3d}", hi);
+		os << std::format("{:3d}", hi);
 	os << "    LADDERS PER SHEET                ." << std::endl;
 
 	// per residue information
@@ -214,7 +216,7 @@ void writeDSSP(const dssp &dssp, std::ostream &os)
 		// can be the transition to a different chain, or missing residues in the current chain
 
 		if (ri.nr() != last + 1)
-			os << cif::format("{:5d}        !{:1c}             0   0    0      0, 0.0     0, 0.0     0, 0.0     0, 0.0   0.000 360.0 360.0 360.0 360.0    0.0    0.0    0.0",
+			os << std::format("{:5d}        !{:1c}             0   0    0      0, 0.0     0, 0.0     0, 0.0     0, 0.0   0.000 360.0 360.0 360.0 360.0    0.0    0.0    0.0",
 					  (last + 1), (ri.chain_break() == dssp::chain_break_type::NewChain ? '*' : ' '))
 			   << std::endl;
 
@@ -329,9 +331,8 @@ void writeSheets(cif::datablock &db, const dssp &dssp)
 	// clean up old info first
 
 	for (auto sheet_cat : { "struct_sheet", "struct_sheet_order", "struct_sheet_range", "struct_sheet_hbond", "pdbx_struct_sheet_hbond" })
-		db.erase(remove_if(db.begin(), db.end(), [sheet_cat](const cif::category &cat)
-					 { return cat.name() == sheet_cat; }),
-			db.end());
+		std::erase_if(db, [sheet_cat](const cif::category &cat)
+			{ return cat.name() == sheet_cat; });
 
 	// create a list of strands, based on the SS info in DSSP. Store sheet number along with the strand.
 
@@ -370,7 +371,7 @@ void writeSheets(cif::datablock &db, const dssp &dssp)
 
 			std::string strandID = cif::cif_id_for_number(strand.front().strand() - 1);
 
-			std::sort(strand.begin(), strand.end(), [](dssp::residue_info const &a, dssp::residue_info const &b)
+			std::ranges::sort(strand, [](dssp::residue_info const &a, dssp::residue_info const &b)
 				{ return a.nr() < b.nr(); });
 
 			auto &beg = strand.front();
@@ -439,7 +440,7 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 
 				assert(l.parallel == parallel);
 
-				if (find_if(l.pairs.begin(), l.pairs.end(), [na = p.nr(), nb = res.nr()](const auto &p)
+				if (std::ranges::find_if(l.pairs, [na = p.nr(), nb = res.nr()](const auto &p)
 						{ return p.first.nr() == na and p.second.nr() == nb; }) != l.pairs.end())
 				{
 					is_new = false;
@@ -458,7 +459,7 @@ void writeLadders(cif::datablock &db, const dssp &dssp)
 		}
 	}
 
-	std::sort(ladders.begin(), ladders.end());
+	std::ranges::sort(ladders, std::less<>());
 
 	auto &dssp_struct_ladder = db["dssp_struct_ladder"];
 
@@ -545,7 +546,7 @@ void writeStatistics(cif::datablock &db, const dssp &dssp)
 
 	for (int k = 0; k < 11; ++k)
 		dssp_struct_hbonds.emplace({ { "entry_id", db.name() },
-			{ "type", "O(I)-->H-N(I"s + char(k - 5 < 0 ? '-' : '+') + std::to_string(abs(k - 5)) + ")" },
+			{ "type", "O(I)-->H-N(I"s + (k - 5 < 0 ? '-' : '+') + std::to_string(abs(k - 5)) + ")" },
 			{ "count", stats.count.H_Bonds_per_distance[k] },
 			{ "count_per_100", stats.count.H_Bonds_per_distance[k] * 100.0 / stats.count.residues, 1 } });
 
@@ -737,6 +738,17 @@ void writeSummary(cif::datablock &db, const dssp &dssp)
 	}
 }
 
+// Work around stupid change in signature of cif::validator_factory::get...
+template <typename ValidatorFactory>
+void fill_audit_conform(cif::category &audit_conform)
+{
+	auto &cf = ValidatorFactory::instance();
+	if constexpr (std::is_pointer_v<decltype(std::declval<cif::validator_factory>().get(""))>)
+		cf.get("mmcif_pdbx.dic")->fill_audit_conform(audit_conform);
+	else
+		cf.get("mmcif_pdbx.dic").fill_audit_conform(audit_conform);
+}
+
 void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool writeNewFormat)
 {
 	using namespace std::literals;
@@ -745,8 +757,8 @@ void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool wr
 
 	if (audit_conform.empty())
 	{
-		auto &cf = cif::validator_factory::instance();
-		cf.get("mmcif_pdbx.dic").fill_audit_conform(audit_conform);
+		static_assert(std::is_reference_v<decltype(std::declval<cif::validator_factory>().get(""))>);
+		fill_audit_conform<cif::validator_factory>(audit_conform);
 	}
 
 	audit_conform.erase(cif::key("dict_name") == "dssp-extension.dic");
@@ -771,10 +783,10 @@ void annotateDSSP(cif::datablock &db, const dssp &dssp, bool writeOther, bool wr
 				 "dssp_statistics",
 				 "dssp_statistics_hbond",
 				 "dssp_statistics_histogram",
-				 "dssp_struct_summary"
-			 })
+				 "dssp_struct_summary" })
 		{
-			db.erase(std::remove_if(db.begin(), db.end(), [cat] (cif::category &c) { return c.name() == cat; }), db.end());
+			std::erase_if(db, [cat](cif::category &c)
+				{ return c.name() == cat; });
 		}
 
 		if (writeNewFormat)
